import { startServerAndCreateNextHandler } from "@as-integrations/next";
import { ApolloServer } from "@apollo/server";
import { NextRequest } from "next/server";
import { gql } from "graphql-tag";
import { prisma } from "@/lib/prisma";

const typeDefs = gql`
  type Query {
    users: [User],
    prisma_okay: String
  }

  type User {
    id: ID!
    name: String
    email: String
  }
`;

const resolvers = {
  Query: {
    users: async () => {
      const db_users = await prisma.user.findMany();
      if (!db_users) return [];
      const map = db_users.map((user) => {
        return {
          id: user.id,
          name: user.name,
          email: user.email,
        };
      });
      return map;
    },
    prisma_okay: async () => {
      try {
        await prisma.$connect();
        return "Prisma is okay";
      } catch (error) {
        return "Prisma is not okay: " + error;
      }
    },
  },
};

const server = new ApolloServer({
    typeDefs,
    resolvers,
});

const handler = startServerAndCreateNextHandler<NextRequest>(server, {
  context: async (req, res) => ({
    req,
    res,
    dataSources: {},
  }),
});

export async function GET(request: NextRequest) {
  return handler(request);
}
export async function POST(request: NextRequest) {
  return handler(request);
}