"use client";

import { Geist, Geist_Mono } from "next/font/google";
import "./globals.css";
import IntroAnim from "@/components/intro_anim";
import React from "react";

const geistSans = Geist({
  variable: "--font-geist-sans",
  subsets: ["latin"],
});

const geistMono = Geist_Mono({
  variable: "--font-geist-mono",
  subsets: ["latin"],
});


export default function RootLayout({
  children,
}: Readonly<{
  children: React.ReactNode;
}>) {
  const [showIntro, setShowIntro] = React.useState(true);
  // Increment this value by deltatime - If it exceeds 0.2 seconds, negate the check
  const [checkExistsTime, setCheckExistsTime] = React.useState(0);
  // Less is sharper, more is blurrier
  const [focus, setFocus] = React.useState(1);
  const [darkness, setDarkness] = React.useState(1);
  const [lastFocusTime, setLastFocusTime] = React.useState(Date.now());
  const [channel] = React.useState(new BroadcastChannel("felidae"));
  // Skip the intro animation if the site is already open on another tab
  React.useEffect(() => {
    channel.onmessage = (event) => {
      if (event.data === "ping" && checkExistsTime > 200) {
        // Broadcast across ALL tabs
        const channel = new BroadcastChannel("felidae");
        channel.postMessage("pong");

        console.log("Received ping message");
      } else if (event.data === "pong" && checkExistsTime < 200) {
        setShowIntro(false);
        console.log("Received pong message");
      }
    };
  });

  // Initial ping to check if the site is already open on another tab
  React.useEffect(() => {
    channel.postMessage("ping");
  });

  // If the user has Reduced Motion enabled, skip the intro animation
  React.useEffect(() => {
    const mediaQuery = window.matchMedia("(prefers-reduced-motion: reduce)");
    if (mediaQuery.matches) {
      setShowIntro(false);
      setFocus(0);
      setDarkness(1);
    }
  }, []);

  // When showIntro is false, reduce pixelation towards zero by a factor of 0.1 every tenth of a second.
  React.useEffect(() => {
    if (!showIntro) {
      setLastFocusTime(Date.now());
      const interval = setInterval(() => {
          const timeSinceLastFocus = Date.now() - lastFocusTime;
          const amount = focus - timeSinceLastFocus * 0.0003 < 0 ? 0 : focus - timeSinceLastFocus * 0.0003;
          setFocus(amount);
          setDarkness(1 - amount);
          setLastFocusTime(Date.now());
          if (focus <= 0) {
            clearInterval(interval);
          }
      }, 30);
      return () => clearInterval(interval);
    } else {
      // Increment the checkExistsTime by the time since the last frame using an interval
      setLastFocusTime(Date.now());
      const interval = setInterval(() => {
        const timeSinceLastFocus = Date.now() - lastFocusTime;
        setCheckExistsTime(checkExistsTime + timeSinceLastFocus);
        setLastFocusTime(Date.now());
        if (checkExistsTime > 200) {
          clearInterval(interval);
        }
      }, 30);
      return () => clearInterval(interval);
    }
  }, [showIntro]);

  return (
    <html lang="en">
      <body
        className={`${geistSans.variable} ${geistMono.variable} antialiased`}
      >
        {showIntro && <IntroAnim onFinish={() => setShowIntro(false)} />}
        {!showIntro && <div>
          {focus > 0 && <div>
            <svg
              xmlns="http://www.w3.org/2000/svg"
              version="1.1"
              width="0"
              height="0"
            >
              <defs>
                <filter id="blur">
                  <feGaussianBlur in="SourceGraphic" stdDeviation={focus * 3} />

                </filter>
                <filter id="darken">
                  <feComponentTransfer>
                    <feFuncA type="linear" slope={darkness} />
                  </feComponentTransfer>
                </filter>
              </defs>
            </svg>
            <div style={{ filter: "url(#blur) url(#darken)" }}>
              {children}
            </div>
          </div>}
          {focus <= 0 && children}
        </div>}
      </body>
    </html>
  );
}
